# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
#
# Copyright 2021 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License version 3 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
from unittest.mock import ANY, Mock, call, patch

import pytest
from craft_store import Auth, base_client, creds, endpoints, errors
from craft_store.store_client import StoreClient, WebBrowserWaitingInteractor
from macaroonbakery import bakery, httpbakery
from pymacaroons.macaroon import Macaroon


def _fake_response(status_code, reason=None, json=None):
    response = Mock(spec="requests.Response")
    response.status_code = status_code
    response.ok = status_code == 200
    response.reason = reason
    if json is not None:
        response.json = Mock(return_value=json)
    return response


@pytest.fixture()
def real_macaroon():
    return json.dumps(
        {
            "s64": "a0Vi7CwhHWjS4bxzKPhCZQIEJDvlbh9FyhOtWx0tNFQ",
            "c": [
                {"i": "time-before 2022-03-18T19:54:57.151721Z"},
                {
                    "v64": "pDqaL9KDrPfCQCLDUdPc8yO2bTQheWGsM1tpxRaS_4BT3r6zpdnT5TelXz8vpjb4iUhTnc60-x5DPKJOpRuwAi4qMdNa67Vo",
                    "l": "https://api.jujucharms.com/identity/",
                    "i64": "AoZh2j7mbDQgh3oK3qMqoXKKFAnJvmOKwmDCNYHIxHqQnFLJZJUBpqoiJtqra-tyXPPMUTmfuXMgOWP7xKwTD26FBgtJBdh1mE1wt3kf0Ur_TnOzbAWQCHKxqK9jAp1jYv-LlLLAlQAmoqvz9fBf2--dIxHiLIRTThmAESAnlLZHOJ7praDmIScsLQC475a85avA",
                },
            ],
            "l": "api.snapcraft.io",
            "i64": "AwoQ2Ft5YBjnovqdr8VNV3TSlhIBMBoOCgVsb2dpbhIFbG9naW4",
        }
    )


@pytest.fixture()
def http_client_request_mock(real_macaroon):
    def request(*args, **kwargs):  # noqa: ARG001
        if args[1] == "POST" and "tokens" in args[2]:
            response = _fake_response(200, json={"macaroon": real_macaroon})
        elif args[1] == "GET" and "whoami" in args[2]:
            response = _fake_response(
                200,
                json={"name": "Fake Person", "username": "fakeuser", "id": "fake-id"},
            )
        elif args[1] == "POST" and args[2] in (
            "https://fake-charm-storage.com/unscanned-upload/",
            "https://fake-snap-storage.com/unscanned-upload/",
        ):
            response = _fake_response(
                200,
                json={"upload_id": "12345", "successful": True},
            )
        else:
            response = _fake_response(200)

        return response

    patched_http_client = patch(
        "craft_store.base_client.HTTPClient.request",
        autospec=True,
        side_effect=request,
    )
    mocked_http_client = patched_http_client.start()
    yield mocked_http_client
    patched_http_client.stop()


@pytest.fixture()
def _bakery_discharge_mock(monkeypatch):
    token_response_mock = _fake_response(
        200, json={"kind": "kind", "token": "TOKEN", "token64": b"VE9LRU42NA=="}
    )
    monkeypatch.setattr(
        httpbakery.Client, "acquire_discharge", lambda: token_response_mock
    )

    def mock_discharge(*args, **kwargs):  # noqa: ARG001
        return [
            Macaroon(
                location="fake-server.com",
                signature="d9533461d7835e4851c7e3b639144406cf768597dea6e133232fbd2385a5c050",
            )
        ]

    monkeypatch.setattr(bakery, "discharge_all", mock_discharge)


@pytest.fixture()
def auth_mock(real_macaroon, new_auth):
    patched_auth = patch("craft_store.base_client.Auth", autospec=True)
    mocked_auth = patched_auth.start()

    wrapped_credentials = creds.marshal_candid_credentials(real_macaroon)
    stored_credentials = wrapped_credentials if new_auth else real_macaroon

    mocked_auth.return_value.get_credentials.return_value = stored_credentials
    mocked_auth.return_value.encode_credentials.return_value = "c2VjcmV0LWtleXM="
    yield mocked_auth
    patched_auth.stop()


@pytest.mark.usefixtures("_bakery_discharge_mock")
@pytest.mark.parametrize("ephemeral_auth", [True, False])
@pytest.mark.parametrize("environment_auth", [None, "APPLICATION_CREDENTIALS"])
def test_store_client_login(
    http_client_request_mock, real_macaroon, auth_mock, environment_auth, ephemeral_auth
):
    store_client = StoreClient(
        base_url="https://fake-server.com",
        storage_base_url="https://fake-storage.com",
        endpoints=endpoints.CHARMHUB,
        application_name="fakecraft",
        user_agent="FakeCraft Unix X11",
        environment_auth=environment_auth,
        ephemeral=ephemeral_auth,
    )

    credentials = store_client.login(
        permissions=["perm-1", "perm-2"], description="fakecraft@foo", ttl=60
    )

    assert credentials == "c2VjcmV0LWtleXM="
    assert http_client_request_mock.mock_calls == [
        call(
            store_client.http_client,
            "POST",
            "https://fake-server.com/v1/tokens",
            headers={"Content-Type": "application/json", "Accept": "application/json"},
            json={
                "permissions": ["perm-1", "perm-2"],
                "description": "fakecraft@foo",
                "ttl": 60,
            },
        ),
        call(
            store_client.http_client,
            "POST",
            "https://fake-server.com/v1/tokens/exchange",
            headers={
                "Macaroons": "W3siaWRlbnRpZmllciI6ICIiLCAic2lnbmF0dXJlIjogImQ5NTMzNDYxZDc4MzVlNDg1MWM3ZTNiNjM5MTQ0NDA2Y2Y3Njg1OTdkZWE2ZTEzMzIzMmZiZDIzODVhNWMwNTAiLCAibG9jYXRpb24iOiAiZmFrZS1zZXJ2ZXIuY29tIn1d"
            },
            json={},
        ),
    ]

    wrapped = creds.marshal_candid_credentials(real_macaroon)

    assert auth_mock.mock_calls == [
        call(
            "fakecraft",
            "fake-server.com",
            environment_auth=environment_auth,
            ephemeral=ephemeral_auth,
        ),
        call().ensure_no_credentials(),
        call().set_credentials(wrapped),
        call().encode_credentials(wrapped),
    ]


@pytest.mark.usefixtures("_bakery_discharge_mock")
def test_store_client_login_with_packages_and_channels(
    http_client_request_mock, real_macaroon, auth_mock
):
    store_client = StoreClient(
        base_url="https://fake-server.com",
        storage_base_url="https://fake-storage.com",
        endpoints=endpoints.CHARMHUB,
        application_name="fakecraft",
        user_agent="FakeCraft Unix X11",
    )

    credentials = store_client.login(
        permissions=["perm-1", "perm-2"],
        description="fakecraft@foo",
        ttl=60,
        channels=["edge"],
        packages=[
            endpoints.Package("my-charm", "charm"),
            endpoints.Package("my-bundle", "bundle"),
        ],
    )

    assert credentials == "c2VjcmV0LWtleXM="
    assert http_client_request_mock.mock_calls == [
        call(
            store_client.http_client,
            "POST",
            "https://fake-server.com/v1/tokens",
            headers={"Content-Type": "application/json", "Accept": "application/json"},
            json={
                "permissions": ["perm-1", "perm-2"],
                "description": "fakecraft@foo",
                "ttl": 60,
                "packages": [
                    {
                        "name": "my-charm",
                        "type": "charm",
                    },
                    {
                        "name": "my-bundle",
                        "type": "bundle",
                    },
                ],
                "channels": ["edge"],
            },
        ),
        call(
            store_client.http_client,
            "POST",
            "https://fake-server.com/v1/tokens/exchange",
            headers={
                "Macaroons": "W3siaWRlbnRpZmllciI6ICIiLCAic2lnbmF0dXJlIjogImQ5NTMzNDYxZDc4MzVlNDg1MWM3ZTNiNjM5MTQ0NDA2Y2Y3Njg1OTdkZWE2ZTEzMzIzMmZiZDIzODVhNWMwNTAiLCAibG9jYXRpb24iOiAiZmFrZS1zZXJ2ZXIuY29tIn1d"
            },
            json={},
        ),
    ]

    expected_credentials = creds.marshal_candid_credentials(real_macaroon)

    assert auth_mock.mock_calls == [
        call("fakecraft", "fake-server.com", environment_auth=None, ephemeral=False),
        call().ensure_no_credentials(),
        call().set_credentials(expected_credentials),
        call().encode_credentials(expected_credentials),
    ]


def test_store_client_logout(auth_mock):
    store_client = StoreClient(
        base_url="https://fake-server.com",
        storage_base_url="https://fake-storage.com",
        endpoints=endpoints.CHARMHUB,
        application_name="fakecraft",
        user_agent="FakeCraft Unix X11",
    )

    store_client.logout()

    assert auth_mock.mock_calls == [
        call("fakecraft", "fake-server.com", environment_auth=None, ephemeral=False),
        call().del_credentials(),
    ]


def test_store_client_request(http_client_request_mock, real_macaroon, auth_mock):
    store_client = StoreClient(
        base_url="https://fake-server.com",
        storage_base_url="https://fake-storage.com",
        endpoints=endpoints.CHARMHUB,
        application_name="fakecraft",
        user_agent="FakeCraft Unix X11",
    )

    store_client.request("GET", "https://fake-server.com/fakepath")

    assert http_client_request_mock.mock_calls == [
        call(
            store_client.http_client,
            "GET",
            "https://fake-server.com/fakepath",
            params=None,
            headers={"Authorization": f"Macaroon {real_macaroon}"},
        )
    ]

    assert auth_mock.mock_calls == [
        call("fakecraft", "fake-server.com", environment_auth=None, ephemeral=False),
        call().get_credentials(),
    ]


def test_store_client_whoami(http_client_request_mock, real_macaroon, auth_mock):
    store_client = StoreClient(
        base_url="https://fake-server.com",
        storage_base_url="https://fake-storage.com",
        endpoints=endpoints.CHARMHUB,
        application_name="fakecraft",
        user_agent="FakeCraft Unix X11",
    )

    assert store_client.whoami() == {
        "name": "Fake Person",
        "username": "fakeuser",
        "id": "fake-id",
    }

    assert http_client_request_mock.mock_calls == [
        call(
            store_client.http_client,
            "GET",
            "https://fake-server.com/v1/tokens/whoami",
            params=None,
            headers={"Authorization": f"Macaroon {real_macaroon}"},
        )
    ]

    assert auth_mock.mock_calls == [
        call("fakecraft", "fake-server.com", environment_auth=None, ephemeral=False),
        call().get_credentials(),
    ]


@pytest.mark.parametrize("hub", [endpoints.CHARMHUB, endpoints.SNAP_STORE])
def test_store_client_upload_file_no_monitor(tmp_path, http_client_request_mock, hub):
    if hub == endpoints.CHARMHUB:
        storage_url = "https://fake-charm-storage.com"
    else:
        storage_url = "https://fake-snap-storage.com"

    store_client = StoreClient(
        base_url="https://fake-server.com",
        storage_base_url=storage_url,
        endpoints=hub,
        application_name="fakecraft",
        user_agent="FakeCraft Unix X11",
    )

    filepath = tmp_path / "artifact.thing"
    filepath.touch()

    assert store_client.upload_file(filepath=filepath) == "12345"
    store_client.upload_file(filepath=filepath)
    assert http_client_request_mock.mock_calls == [
        call(
            store_client.http_client,
            "POST",
            f"{storage_url}/unscanned-upload/",
            headers={
                "Content-Type": ANY,
                "Accept": "application/json",
            },
            data=ANY,
        ),
        call(
            store_client.http_client,
            "POST",
            f"{storage_url}/unscanned-upload/",
            headers={
                "Content-Type": ANY,
                "Accept": "application/json",
            },
            data=ANY,
        ),
    ]


@pytest.mark.parametrize("hub", [endpoints.CHARMHUB, endpoints.SNAP_STORE])
def test_store_client_upload_file_with_monitor(tmp_path, http_client_request_mock, hub):
    if hub == endpoints.CHARMHUB:
        storage_url = "https://fake-charm-storage.com"
    else:
        storage_url = "https://fake-snap-storage.com"

    store_client = StoreClient(
        base_url="https://fake-server.com",
        storage_base_url=storage_url,
        endpoints=hub,
        application_name="fakecraft",
        user_agent="FakeCraft Unix X11",
    )

    filepath = tmp_path / "artifact.thing"
    filepath.write_text("file to upload")

    def callback(monitor):  # noqa: ARG001
        pass

    def monitor(encoder):  # noqa: ARG001
        return callback

    with patch("craft_store.base_client.MultipartEncoder"):
        with patch(
            "craft_store.base_client.MultipartEncoderMonitor",
            wraps=base_client.MultipartEncoderMonitor,
        ) as wrapped_encoder_monitor:
            assert (
                store_client.upload_file(filepath=filepath, monitor_callback=monitor)
                == "12345"
            )
            assert wrapped_encoder_monitor.mock_calls == [
                call(
                    base_client.MultipartEncoder(
                        {
                            "binary": (
                                "artifact.thing",
                                ANY,
                                "application/octect-stream",
                            )
                        },
                    ),
                    monitor(monitor),
                )
            ]

    store_client.upload_file(filepath=filepath)
    assert http_client_request_mock.mock_calls == [
        call(
            store_client.http_client,
            "POST",
            f"{storage_url}/unscanned-upload/",
            headers={
                "Content-Type": ANY,
                "Accept": "application/json",
            },
            data=ANY,
        ),
        call(
            store_client.http_client,
            "POST",
            f"{storage_url}/unscanned-upload/",
            headers={
                "Content-Type": ANY,
                "Accept": "application/json",
            },
            data=ANY,
        ),
    ]


def test_webinteractore_wait_for_token(http_client_request_mock):
    http_client_request_mock.side_effect = None
    http_client_request_mock.return_value = _fake_response(
        200, json={"kind": "kind", "token": "TOKEN", "token64": b"VE9LRU42NA=="}
    )

    wbi = WebBrowserWaitingInteractor(user_agent="foobar")

    discharged_token = wbi._wait_for_token(None, "https://foo.bar/candid")

    assert discharged_token == httpbakery.DischargeToken(kind="kind", value="TOKEN")
    assert http_client_request_mock.mock_calls == [
        call(ANY, "GET", "https://foo.bar/candid")
    ]


def test_webinteractore_wait_for_token_timeout_error(
    http_client_request_mock,
):
    http_client_request_mock.side_effect = None
    http_client_request_mock.return_value = _fake_response(400, json={})

    wbi = WebBrowserWaitingInteractor(user_agent="foobar")

    with pytest.raises(errors.CandidTokenTimeoutError):
        wbi._wait_for_token(None, "https://foo.bar/candid")


def test_webinteractore_wait_for_token_kind_error(http_client_request_mock):
    http_client_request_mock.side_effect = None
    http_client_request_mock.return_value = _fake_response(200, json={})

    wbi = WebBrowserWaitingInteractor(user_agent="foobar")

    with pytest.raises(errors.CandidTokenKindError):
        wbi._wait_for_token(None, "https://foo.bar/candid")


def test_webinteractore_wait_for_token_value_error(http_client_request_mock):
    http_client_request_mock.side_effect = None
    http_client_request_mock.return_value = _fake_response(
        200,
        json={
            "kind": "kind",
        },
    )

    wbi = WebBrowserWaitingInteractor(user_agent="foobar")

    with pytest.raises(errors.CandidTokenValueError):
        wbi._wait_for_token(None, "https://foo.bar/candid")


def test_store_client_env_var(http_client_request_mock, new_auth, monkeypatch):
    """
    Test StoreClient credential handling when using auth from an environment variable.
    """

    # The auth environment variable contents must have the same format as those stored
    # by Auth.set_credentials(): some payload, base64-encoded. So we "manually" encode
    # a serialized dummy macaroon.
    macaroon = Macaroon(
        location="fake-server.com",
        signature="d9533461d7835e4851c7e3b639144406cf768597dea6e133232fbd2385a5c050",
    )
    credentials = macaroon.serialize()

    if new_auth:
        # new, type-tagged auth: use the dict tagging the type and the actual payload
        # (the serialized macaroon).
        wrapped_credentials = creds.marshal_candid_credentials(credentials)
        stored_b64_credentials = Auth.encode_credentials(wrapped_credentials)
    else:
        # old auth: use the serialized macaroon "as-is".
        stored_b64_credentials = Auth.encode_credentials(credentials)

    environment_auth = "CRAFT_AUTH_TEST_VAR"
    monkeypatch.setenv(environment_auth, stored_b64_credentials)

    # This StoreClient uses a non-mocked Auth with a memory keyring because of `environment_auth`.
    store_client = StoreClient(
        base_url="https://fake-server.com",
        storage_base_url="https://fake-storage.com",
        endpoints=endpoints.CHARMHUB,
        application_name="fakecraft",
        user_agent="FakeCraft Unix X11",
        environment_auth=environment_auth,
    )

    # Make a (mocked) network request to check the provided credentials.
    assert store_client.whoami() == {
        "name": "Fake Person",
        "username": "fakeuser",
        "id": "fake-id",
    }

    # When making the request the Authorization header must contain the original
    # serialized (non-base64-encoded) macaroon, regardless of new or old auth.
    assert http_client_request_mock.mock_calls == [
        call(
            store_client.http_client,
            "GET",
            "https://fake-server.com/v1/tokens/whoami",
            params=None,
            headers={"Authorization": f"Macaroon {credentials}"},
        )
    ]



#  -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*
#
#  Copyright 2023-2024 Canonical Ltd.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License version 3 as published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Tests for RegisteredNameModel."""

import pydantic_core
import pytest
from craft_store.models import (
    AccountModel,
    RegisteredNameModel,
    TrackGuardrailModel,
    TrackModel,
)
from craft_store.models.registered_name_model import MediaModel

BASIC_REGISTERED_NAME = {
    "id": "123",
    "private": "true",
    "publisher": {"id": "456"},
    "status": "registered",
    "store": "ubuntu",
    "type": "charm",
}
REGISTERED_NAME_ALL_FIELDS = {
    "authority": "Mark",
    "contact": "charmcrafters@lists.canonical.com",
    "default-track": "lts",
    "description": "This is a thing",
    "id": "123",
    "links": {"Ubuntu": "https://ubuntu.com/"},
    "media": [
        {"type": "icon", "url": "https://assets.ubuntu.com/v1/0843d517-favicon.ico"}
    ],
    "name": "charming-charm",
    "private": "false",
    "publisher": {"id": "456"},
    "status": "registered",
    "store": "ubuntu",
    "summary": "This is a thing",
    "title": "Some charming charm",
    "track-guardrails": [
        {"created-at": "2023-03-28T18:50:44+00:00", "pattern": r"^\d\.\d/"}
    ],
    "tracks": [{"created-at": "2023-03-28T18:50:44+00:00", "name": "1.0/stable"}],
    "type": "charm",
    "website": "https://canonical.com/",
}


@pytest.mark.parametrize(
    "json_dict",
    [
        pytest.param(BASIC_REGISTERED_NAME, id="basic"),
        pytest.param(REGISTERED_NAME_ALL_FIELDS, id="all_fields"),
    ],
)
def test_unmarshal(check, json_dict):
    actual = RegisteredNameModel.unmarshal(json_dict)

    check.equal(actual.authority, json_dict.get("authority"))
    check.equal(actual.contact, json_dict.get("contact"))
    check.equal(actual.default_track, json_dict.get("default-track"))
    check.equal(actual.description, json_dict.get("description"))
    check.equal(actual.id, json_dict.get("id"))
    check.equal(actual.links, json_dict.get("links", {}))
    check.equal(
        actual.media, [MediaModel.unmarshal(m) for m in json_dict.get("media", [])]
    )
    check.equal(actual.name, json_dict.get("name"))
    check.equal(actual.private, json_dict["private"] == "true")
    check.equal(actual.publisher, AccountModel.unmarshal(json_dict["publisher"]))
    check.equal(actual.status, json_dict.get("status"))
    check.equal(actual.store, json_dict.get("store"))
    check.equal(actual.summary, json_dict.get("summary"))
    check.equal(actual.title, json_dict.get("title"))
    check.equal(
        actual.tracks, [TrackModel.unmarshal(t) for t in json_dict.get("tracks", [])]
    )
    check.equal(actual.type, json_dict.get("type"))
    if actual.website is None:
        check.is_none(json_dict.get("website"))
    else:
        check.equal(actual.website, pydantic_core.Url(json_dict.get("website")))
    check.equal(
        actual.track_guardrails,
        [
            TrackGuardrailModel.unmarshal(g)
            for g in json_dict.get("track-guardrails", [])
        ],
    )


@pytest.mark.parametrize(
    "payload",
    [
        pytest.param(BASIC_REGISTERED_NAME, id="basic"),
        pytest.param(REGISTERED_NAME_ALL_FIELDS, id="all_fields"),
    ],
)
def test_unmarshal_and_marshal(payload, check):
    marshalled = RegisteredNameModel.unmarshal(payload).marshal()
    not_set = [[["NOT SET"]]]

    check.equal(marshalled.keys(), payload.keys())

    for field in payload:
        actual = marshalled.get(field, not_set)
        expected = payload.get(field, not_set)
        if field == "private":
            expected = payload[field].lower() == "true"
        elif field in ("track-guardrails", "tracks"):
            expected = payload[field].copy()
        check.equal(actual, expected)



#  -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*
#
#  Copyright 2023-2024 Canonical Ltd.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License version 3 as published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""Tests for TrackGuardrailModel"""
import re
from datetime import datetime, timezone

import pytest
from craft_store.models.track_guardrail_model import TrackGuardrailModel

GUARDRAIL_DICT = {"created-at": "2023-03-28T18:50:44+00:00", "pattern": r"^\d\.\d/"}


@pytest.mark.parametrize(
    ("json_dict", "expected"),
    [
        pytest.param(
            GUARDRAIL_DICT,
            TrackGuardrailModel(
                pattern=re.compile(r"^\d\.\d/"),
                **{
                    "created-at": datetime(
                        2023, 3, 28, 18, 50, 44, tzinfo=timezone.utc
                    ),
                },
            ),
        ),
    ],
)
def test_unmarshal(json_dict, expected):
    actual = TrackGuardrailModel.unmarshal(json_dict)

    assert actual == expected


@pytest.mark.parametrize("payload", [GUARDRAIL_DICT])
def test_unmarshal_and_marshal(payload, check):
    marshalled = TrackGuardrailModel.unmarshal(payload).marshal()

    check.equal(payload["pattern"], marshalled["pattern"])
    check.equal(payload["created-at"], marshalled["created-at"])



# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
#
# Copyright 2023 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License version 3 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Tests for list_releases."""
import datetime
from typing import cast

from craft_store.models.resource_revision_model import (
    CharmResourceRevision,
)

from .conftest import needs_charmhub_credentials


@needs_charmhub_credentials()
def test_charm_list_resource_revisions(charm_client, charmhub_charm_name):
    revisions = charm_client.list_resource_revisions(charmhub_charm_name, "empty-file")

    assert len(revisions) >= 1
    assert isinstance(revisions[-1], CharmResourceRevision)

    actual = cast(CharmResourceRevision, revisions[-1])

    # Greater than or equal to in order to allow someone to replicate this
    # integration test themselves.
    assert actual.created_at >= datetime.datetime(
        2023, 12, 1, tzinfo=datetime.timezone.utc
    )
    assert actual.revision >= 1

    sha256s = [r.sha256 for r in revisions]
    sha384s = [r.sha384 for r in revisions]
    sha3_384s = [r.sha3_384 for r in revisions]
    sha512s = [r.sha512 for r in revisions]

    expected_sha256 = "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855"
    expected_sha384 = "38b060a751ac96384cd9327eb1b1e36a21fdb71114be07434c0cc7bf63f6e1da274edebfe76f65fbd51ad2f14898b95b"
    expected_sha3_384 = "0c63a75b845e4f7d01107d852e4c2485c51a50aaaa94fc61995e71bbee983a2ac3713831264adb47fb6bd1e058d5f004"
    expected_sha512 = "cf83e1357eefb8bdf1542850d66d8007d620e4050b5715dc83f4a921d36ce9ce47d0d13c5d85f2b0ff8318d2877eec2f63b931bd47417a81a538327af927da3e"

    assert expected_sha256 in sha256s
    assert expected_sha384 in sha384s
    assert expected_sha3_384 in sha3_384s
    assert expected_sha512 in sha512s
